import { RequestState } from '@samwise-tech/core';
import * as uuid from 'uuid';
import { DISPATCH } from '../reducer-dispatcher/tokens';
import { AsyncOperationsDispatcher } from './dispatcher';
const getAction = 
// tslint:disable-next-line:readonly-array
(id, name, args) => ({
    args,
    id,
    name,
    type: AsyncOperationsDispatcher.ACTION,
});
const getStartAction = (base) => (Object.assign({}, base, { status: RequestState.Start }));
const getFailAction = (base, reason) => (Object.assign({}, base, { reason, status: RequestState.Fail }));
const getDoneAction = (base, data) => (Object.assign({}, base, { data, status: RequestState.Success }));
export const asyncFactory = (dispatcher) => (name, isWrappedEither = true) => (_targetProto, propName, descriptor) => {
    const original = descriptor.value;
    // tslint:disable-next-line:no-object-mutation
    descriptor.value = function (...args) {
        const action = getAction(`${propName}:async:${uuid.v4()}`, name, args);
        dispatcher[DISPATCH](getStartAction(action));
        const promise = original.apply(this, args);
        if (isWrappedEither) {
            promise.then((resolved) => {
                resolved.cata(reason => {
                    dispatcher[DISPATCH](getFailAction(action, reason));
                }, response => {
                    dispatcher[DISPATCH](getDoneAction(action, response));
                });
            }, (reason) => {
                dispatcher[DISPATCH](getFailAction(action, reason));
            });
        }
        else {
            promise.then((response) => {
                dispatcher[DISPATCH](getDoneAction(action, response));
            }, (reason) => {
                dispatcher[DISPATCH](getFailAction(action, reason));
            });
        }
        return promise;
    };
};
export const async = asyncFactory(AsyncOperationsDispatcher.instance());
//# sourceMappingURL=async.js.map