import { RequestState } from '@samwise-tech/core';
import { Reducer } from '../reducer-dispatcher/reducer';
import { DISPATCH } from '../reducer-dispatcher/tokens';
import { AsyncOperationsState } from './state';
export class AsyncOperationsDispatcher extends Reducer {
    constructor() {
        super(AsyncOperationsState.empty());
        this.reduce = this.reduce.bind(this);
    }
    static instance() {
        return this.__instance;
    }
    removeFailed(id) {
        this[DISPATCH]({
            id,
            status: RequestState.Fail,
            type: AsyncOperationsDispatcher.REMOVE_ACTION,
        });
    }
    removePending(id) {
        this[DISPATCH]({
            id,
            status: RequestState.Start,
            type: AsyncOperationsDispatcher.REMOVE_ACTION,
        });
    }
    reduce(state = this.state, action) {
        return this.isAsyncAction(action) ?
            this.reduceAsyncOperations(state, action) :
            this.isRemoveAction(action) ?
                this.reduceRemoveOperations(state, action) :
                state;
    }
    reduceAsyncOperations(state, action) {
        switch (action.status) {
            case RequestState.Start: return state.start(action);
            case RequestState.Fail: return state.fail(action);
            case RequestState.Success: return state.done(action);
            default: return state;
        }
    }
    reduceRemoveOperations(state, action) {
        switch (action.status) {
            case RequestState.Start: return state.removePending(action.id);
            case RequestState.Fail: return state.removeFailed(action.id);
            default: return state;
        }
    }
    isAsyncAction(a) {
        return Boolean(a) && a.type === AsyncOperationsDispatcher.ACTION;
    }
    isRemoveAction(a) {
        return Boolean(a) && a.type === AsyncOperationsDispatcher.REMOVE_ACTION;
    }
}
AsyncOperationsDispatcher.ACTION = '@SAMWISE-TECH/REDUX/ASYNC';
AsyncOperationsDispatcher.REMOVE_ACTION = '@SAMWISE-TECH/REDUX/ASYNC_REMOVE';
AsyncOperationsDispatcher.__instance = new AsyncOperationsDispatcher();
export const asyncDispatcher = new AsyncOperationsDispatcher();
//# sourceMappingURL=dispatcher.js.map