"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var core_1 = require("@samwise-tech/core");
var reducer_1 = require("../reducer-dispatcher/reducer");
var tokens_1 = require("../reducer-dispatcher/tokens");
var state_1 = require("./state");
var AsyncOperationsDispatcher = /** @class */ (function (_super) {
    tslib_1.__extends(AsyncOperationsDispatcher, _super);
    function AsyncOperationsDispatcher() {
        var _this = _super.call(this, state_1.AsyncOperationsState.empty()) || this;
        _this.reduce = _this.reduce.bind(_this);
        return _this;
    }
    AsyncOperationsDispatcher.instance = function () {
        return this.__instance;
    };
    AsyncOperationsDispatcher.prototype.removeFailed = function (id) {
        this[tokens_1.DISPATCH]({
            id: id,
            status: core_1.RequestState.Fail,
            type: AsyncOperationsDispatcher.REMOVE_ACTION,
        });
    };
    AsyncOperationsDispatcher.prototype.removePending = function (id) {
        this[tokens_1.DISPATCH]({
            id: id,
            status: core_1.RequestState.Start,
            type: AsyncOperationsDispatcher.REMOVE_ACTION,
        });
    };
    AsyncOperationsDispatcher.prototype.reduce = function (state, action) {
        if (state === void 0) { state = this.state; }
        return this.isAsyncAction(action) ?
            this.reduceAsyncOperations(state, action) :
            this.isRemoveAction(action) ?
                this.reduceRemoveOperations(state, action) :
                state;
    };
    AsyncOperationsDispatcher.prototype.reduceAsyncOperations = function (state, action) {
        switch (action.status) {
            case core_1.RequestState.Start: return state.start(action);
            case core_1.RequestState.Fail: return state.fail(action);
            case core_1.RequestState.Success: return state.done(action);
            default: return state;
        }
    };
    AsyncOperationsDispatcher.prototype.reduceRemoveOperations = function (state, action) {
        switch (action.status) {
            case core_1.RequestState.Start: return state.removePending(action.id);
            case core_1.RequestState.Fail: return state.removeFailed(action.id);
            default: return state;
        }
    };
    AsyncOperationsDispatcher.prototype.isAsyncAction = function (a) {
        return Boolean(a) && a.type === AsyncOperationsDispatcher.ACTION;
    };
    AsyncOperationsDispatcher.prototype.isRemoveAction = function (a) {
        return Boolean(a) && a.type === AsyncOperationsDispatcher.REMOVE_ACTION;
    };
    AsyncOperationsDispatcher.ACTION = '@SAMWISE-TECH/REDUX/ASYNC';
    AsyncOperationsDispatcher.REMOVE_ACTION = '@SAMWISE-TECH/REDUX/ASYNC_REMOVE';
    AsyncOperationsDispatcher.__instance = new AsyncOperationsDispatcher();
    return AsyncOperationsDispatcher;
}(reducer_1.Reducer));
exports.AsyncOperationsDispatcher = AsyncOperationsDispatcher;
exports.asyncDispatcher = new AsyncOperationsDispatcher();
//# sourceMappingURL=dispatcher.js.map